﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using HeuristicLab.DataImporter.Data.CommandBase;
using HeuristicLab.DataImporter.Data.Model;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataImporter.DataProcessor.Command {
  [StorableClass]
  public class ReorderColumnsCommand : DataSetCommandBase {
    [Storable]
    private Dictionary<string, int[]> displayedIndexes;

    [StorableConstructor]
    protected ReorderColumnsCommand(bool deserializing) : base(deserializing) { }
    public ReorderColumnsCommand(DataSet ds, Dictionary<string, int[]> displayedIndexes)
      : base(ds) {
      this.displayedIndexes = displayedIndexes;
    }

    public override string Description {
      get { return "Reorder Columns"; }
    }

    public override void Execute() {
      base.Execute();
      ReorderColumns(true);
      this.DataSet.FireChanged();
    }

    public override void UndoExecute() {
      base.UndoExecute();
      ReorderColumns(false);
      this.DataSet.FireChanged();
    }

    private void ReorderColumns(bool forward) {
      ColumnGroup grp;
      int[] indices;
      List<int> sortedColumnIndices;
      int newIndex;
      int oldIndex;
      List<ColumnBase> columns;
      foreach (KeyValuePair<string, int[]> pair in displayedIndexes) {
        grp = this.DataSet.GetColumnGroup(pair.Key);
        indices = pair.Value;
        columns = grp.Columns.ToList();
        sortedColumnIndices = new List<int>(grp.SortedColumnIndexes);
        for (int j = 0; j < indices.Length; j++) {
          sortedColumnIndices = new List<int>(grp.SortedColumnIndexes);
          if (forward) {
            newIndex = j;
            oldIndex = indices[j];
          } else {
            newIndex = indices[j];
            oldIndex = j;
          }
          grp.ReplaceColumn(newIndex, columns[oldIndex]);
          grp.SortedColumnIndexes = sortedColumnIndices;
          //corrects the SortedColumnIndices if sorted colum got reordered
          if (grp.SortedColumnIndexes.Contains(oldIndex)) {
            int insertPos = sortedColumnIndices.IndexOf(oldIndex);
            sortedColumnIndices.Remove(oldIndex);
            sortedColumnIndices.Insert(insertPos, newIndex);
          }
        }
        grp.SortedColumnIndexes = sortedColumnIndices;
        grp.FireChanged();
      }
    }
  }
}
